/*
 * Copyright (c) 2014-2020 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0

KeyframableFilter {
    property string fillProperty
    property string distortProperty
    property string legacyRectProperty: null
    property string rectProperty
    property string valignProperty
    property string halignProperty
    property string backgroundProperty
    property rect filterRect
    property string startValue: '_shotcut:startValue'
    property string middleValue: '_shotcut:middleValue'
    property string endValue:  '_shotcut:endValue'

    width: 350
    height: 200
    
    keyframableParameters: ['transition.fix_rotate_x']
    startValues: [0.0]
    middleValues: [0.0]
    endValues: [0.0]

    Component.onCompleted: {
        filter.blockSignals = true
        filter.set(middleValue, Qt.rect(0, 0, profile.width, profile.height))
        filter.set(startValue, Qt.rect(0, 0, profile.width, profile.height))
        filter.set(endValue, Qt.rect(0, 0, profile.width, profile.height))
        if (filter.isNew) {
            // Add default preset.
            filter.set('transition.fix_rotate_x', 0)
            filter.set(rectProperty,   '0%/0%:100%x100%')
            filter.set(fillProperty, 1)
            filter.set(distortProperty, 0)
            filter.set(valignProperty, 'center')
            filter.set(halignProperty, 'middle')
            if (backgroundProperty)
                filter.set(backgroundProperty, 'color:#00000000')
            filter.savePreset(preset.parameters)
        } else {
            if (legacyRectProperty !== null) {
                var old = filter.get(legacyRectProperty)
                if (old && old.length > 0) {
                    filter.resetProperty(legacyRectProperty)
                    filter.set(rectProperty, old)
                }
            }
            filter.set(middleValue, filter.getRect(rectProperty, filter.animateIn + 1))
            if (filter.animateIn > 0)
                filter.set(startValue, filter.getRect(rectProperty, 0))
            if (filter.animateOut > 0)
                filter.set(endValue, filter.getRect(rectProperty, filter.duration - 1))
        }
        filter.blockSignals = false
        setControls()
        setKeyframedControls()
        if (filter.isNew) {
            filter.set(rectProperty, filter.getRect(rectProperty))
        }
    }

    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }

    function setFilter(position) {
    	if (position !== null) {
            filter.blockSignals = true
            if (position <= 0 && filter.animateIn > 0)
                filter.set(startValue, filterRect)
            else if (position >= filter.duration - 1 && filter.animateOut > 0)
                filter.set(endValue, filterRect)
            else
                filter.set(middleValue, filterRect)
            filter.blockSignals = false
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty(rectProperty)
            positionKeyframesButton.checked = false
            if (filter.animateIn > 0) {
                filter.set(rectProperty, filter.getRect(startValue), 1.0, 0)
                filter.set(rectProperty, filter.getRect(middleValue), 1.0, filter.animateIn - 1)
            }
            if (filter.animateOut > 0) {
                filter.set(rectProperty, filter.getRect(middleValue), 1.0, filter.duration - filter.animateOut)
                filter.set(rectProperty, filter.getRect(endValue), 1.0, filter.duration - 1)
            }
        } else if (!positionKeyframesButton.checked) {
            filter.resetProperty(rectProperty)
            filter.set(rectProperty, filter.getRect(middleValue))
        } else if (position !== null) {
            filter.set(rectProperty, filterRect, 1.0, position)
        }
    }
    
    function setControls() {
   	    if (backgroundProperty) {
            var s = filter.get(backgroundProperty)
            if (s.substring(0, 6) === 'color:')
                bgColor.value = s.substring(6)
            else if  (s.substring(0, 7) === 'colour:')
                bgColor.value = s.substring(7)
        }
        var position = getPosition()
        blockUpdate = true
        rotationSlider.value = filter.getDouble('transition.fix_rotate_x', position)
        rotationSlider.enabled = isSimpleKeyframesActive()
        blockUpdate = false
    }

    function setKeyframedControls() {
        var position = getPosition()
        var newValue = filter.getRect(rectProperty, position)
        if (filterRect !== newValue) {
        	blockCreate.text = 1
            filterRect = newValue
            rectX.value = filterRect.x.toFixed()
            rectY.value = filterRect.y.toFixed()
            rectW.value = filterRect.width.toFixed()
            rectH.value = filterRect.height.toFixed()
            xc.text = (rectX.value + filterRect.width / 2).toFixed(1)
            yc.text = (rectY.value + filterRect.height / 2).toFixed(1)
            scale.value = rectW.value / profile.width * 100
            blockCreate.text = 0
        }
        var enabled = position <= 0 || (position >= (filter.animateIn - 1) && position <= (filter.duration - filter.animateOut)) || position >= (filter.duration - 1)
        rectX.enabled = enabled
        rectY.enabled = enabled
        rectW.enabled = enabled
        rectH.enabled = enabled
    }
    
    function updateSimpleKeyframes() {
        updateFilter('transition.fix_rotate_x', rotationSlider.value, rotationKeyframesButton)
        //updateFilterScale(null)
    }

    GridLayout {
        columns: 6
        anchors.fill: parent
        anchors.margins: 8
        
        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }
        Preset {
            id: preset
            parameters: [fillProperty, distortProperty, rectProperty, halignProperty, valignProperty]
            Layout.columnSpan: 5
            onBeforePresetLoaded: {
                filter.resetProperty(rectProperty)
            }
            onPresetSelected: {
                setControls()
                setKeyframedControls()
                positionKeyframesButton.checked = filter.keyframeCount(rectProperty) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
                filter.blockSignals = true
                filter.set(middleValue, filter.getRect(rectProperty, filter.animateIn + 1))
                if (filter.animateIn > 0)
                    filter.set(startValue, filter.getRect(rectProperty, 0))
                if (filter.animateOut > 0)
                    filter.set(endValue, filter.getRect(rectProperty, filter.duration - 1))
                filter.blockSignals = false
            }
        }
        Label {
            text: qsTr('Position')
            Layout.alignment: Qt.AlignRight
        }
        RowLayout {
            Layout.columnSpan: 3
            Label { 
            	Layout.preferredWidth : 30
            	horizontalAlignment: Text.AlignRight
            	text: 'X' 
            }
            SpinBox {
                id: rectX
                Layout.minimumWidth: 100
                minimumValue: - 10 * profile.width - 10
                maximumValue: profile.width + 10
                decimals: 0
                stepSize: 1
                Keys.onEnterPressed: preset.forceActiveFocus()
                onValueChanged: {
                	if (Math.abs(filterRect.x - value) >= 1) filterRect.x = value
                	if (hovered || activeFocus) xc.text = value + filterRect.width / 2
                	if (blockCreate.text == 0) setFilter(getPosition())
                }
            }
            Label { 
            	Layout.preferredWidth : 30
            	horizontalAlignment: Text.AlignRight
            	text: 'Y' 
            }
            SpinBox {
                id: rectY
                Layout.minimumWidth: 100
                minimumValue: - 10 * profile.height - 10
                maximumValue: profile.height + 10
                decimals: 0
                stepSize: 1
                Keys.onEnterPressed: preset.forceActiveFocus()
                onValueChanged: {
                	if (Math.abs(filterRect.y - value) >= 1) filterRect.y = value
                	if (hovered || activeFocus) yc.text = value + filterRect.height / 2
                	if (blockCreate.text == 0) setFilter(getPosition())
                }
            }
        }
        UndoButton {
            onClicked: {
                rectX.value = rectY.value = 0
                filterRect.x = filterRect.y = 0
                xc.text = filterRect.width / 2
                yc.text = filterRect.height / 2
                setFilter(getPosition())
            }
        }
        KeyframesButton {
            id: positionKeyframesButton
            Layout.rowSpan: 3
            checked: filter.keyframeCount(rectProperty) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
            onToggled: {
                if (checked) {
                    //filter.clearSimpleAnimation(rectProperty)
                    filter.set(rectProperty, filterRect, 1.0, getPosition())
                } else {
                    filter.resetProperty(rectProperty)
                    filter.set(rectProperty, filterRect)
                }
                checked = filter.keyframeCount(rectProperty) > 0 && filter.animateIn <= 0 && filter.animateOut <= 0
            }
        }
        Label {
            text: qsTr('Size')
            Layout.alignment: Qt.AlignRight
        }
        RowLayout {
            Layout.columnSpan: 3
            Label { 
            	Layout.preferredWidth : 30
            	horizontalAlignment: Text.AlignRight
            	text: 'W' 
            }
            SpinBox {
                id: rectW
                Layout.minimumWidth: 100
                minimumValue: 0
               	maximumValue: 10 * profile.width
                decimals: 0
                stepSize: 1
                Keys.onEnterPressed: preset.forceActiveFocus()
                onValueChanged: {
                	if ((hovered || activeFocus) && Math.abs(filterRect.width - value) >= 1) {
                		filterRect.width = value
                    	rectX.value = xc.text - filterRect.width / 2
                    	filterRect.height = rectH.value = Math.round( value * profile.height / profile.width )
                		rectY.value = yc.text - filterRect.height / 2
                    	setFilter(getPosition())
                		scale.value = rectW.value / profile.width * 100
                	}
                }
            }
    		Label { 
            	Layout.preferredWidth : 30
            	horizontalAlignment: Text.AlignRight
            	text: 'H' 
            }
            SpinBox {
                id: rectH
                Layout.minimumWidth: 100
                minimumValue: 0
               	maximumValue: 10 * profile.height
                decimals: 0
                stepSize: 1
                Keys.onEnterPressed: preset.forceActiveFocus()
                onValueChanged: {
                	if ((hovered || activeFocus) && Math.abs(filterRect.height - value) >= 1) {
                		filterRect.height = value
                		rectY.value = yc.text - filterRect.height / 2
                    	filterRect.width = rectW.value = Math.round(value * profile.width / profile.height)
                    	rectX.value = xc.text - filterRect.width / 2
                		setFilter(getPosition())
                		scale.value = rectH.value / profile.height * 100
                	}
                }
            }
        }
        Item { 
        	Layout.fillWidth: true
        }
        Label {
            text: qsTr('Scale') 
            Layout.alignment: Qt.AlignRight
        }
        RowLayout {
            Layout.columnSpan: 3
        	SliderSpinner {
            	id: scale
            	minimumValue: 0
            	maximumValue: 1000
            	value: rectW.value / profile.width * 100
            	decimals: 1
            	spinnerWidth: 90
            	suffix: ' %'
            	onValueChanged: {
                    if (Math.abs(value - rectW.value * 100 / profile.width) > 0.1) {
                    	filterRect.width = rectW.value = Math.round(profile.width * value / 100)
                    	filterRect.height = rectH.value = Math.round(profile.height * value / 100)
                    	rectX.value = xc.text - filterRect.width / 2
                		rectY.value = yc.text - filterRect.height / 2
                		setFilter(getPosition())
                	}
                }
            }
        }
        UndoButton {
            onClicked: scale.value = 100
        }
        RowLayout {
            Layout.columnSpan: 6
            Item { Layout.fillWidth: true }
        }
            
        Label {
            text: qsTr('Rotation')
            Layout.alignment: Qt.AlignRight
        }
        RowLayout {
            Layout.columnSpan: 3
        	SliderSpinner {
            	id: rotationSlider
            	minimumValue: -360
            	maximumValue: 360
            	decimals: 1
            	spinnerWidth: 90
            	suffix: qsTr(' °', 'degrees')
            	onValueChanged: updateFilter('transition.fix_rotate_x', value, rotationKeyframesButton, getPosition())
        	}
        }
        	UndoButton {
            	onClicked: rotationSlider.value = 0
        	}
        	KeyframesButton {
            	id: rotationKeyframesButton
            	checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('transition.fix_rotate_x') > 0
            	onToggled: {
                	toggleKeyframes(checked, 'transition.fix_rotate_x', rotationSlider.value)
                	setControls()
            	}
        	}
        
        RowLayout { // pour créer un espace avant la ligne
            Layout.columnSpan: 6
            Item { 
            	Layout.preferredHeight : 10
            	Layout.fillWidth: true 
            }
        }
        
        Label {
            text: qsTr('Background color')
            Layout.alignment: Qt.AlignRight
            visible: bgColor.visible
        }
        ColorPicker {
            id: bgColor
            visible: !!backgroundProperty
            Layout.columnSpan: 3
            eyedropper: true
            alpha: true
            onValueChanged: filter.set(backgroundProperty, 'color:' + value)
        }
        UndoButton {
            visible: bgColor.visible
            onClicked: bgColor.value = '#00000000'
        }
        // Coordonnée x du centre (invisible)
        TextField {
        	id: xc
        	visible: false 
        }
        // Coordonnée y du centre (invisible)
        TextField {
        	id: yc
        	visible: false
        }
        // Blocage création d'images clés lors de la lecture (invisible)
        TextField { 
        	id: blockCreate
        	visible: false
        }
        Item {
            Layout.fillWidth: true
            visible: bgColor.visible
        }
        Item { Layout.fillHeight: true }
    }

    Connections {
        target: filter
        onChanged: {
        	setKeyframedControls()
        }
        onInChanged: updateSimpleKeyframes()
        onOutChanged: updateSimpleKeyframes()
        onAnimateInChanged: updateSimpleKeyframes()
        onAnimateOutChanged: updateSimpleKeyframes()
    }

    Connections {
        target: producer
        onPositionChanged: {
        	setKeyframedControls()
        	setControls()
        }
    }
// Ajout nécessaire ?    
    Connections {
        target: parameters
        onKeyframeAdded: {
            var n = filter.getDouble(parameter, position)
            filter.set(parameter, n, position)
        }
    }
}
